#!/usr/bin/tclsh
# Part of MCU 8051 IDE ( http://mcu8051ide.sf.net )

############################################################################
#    Copyright (C) 2007-2009 by Martin Ošmera                              #
#    martin.osmera@gmail.com                                               #
#                                                                          #
#    This program is free software; you can redistribute it and#or modify  #
#    it under the terms of the GNU General Public License as published by  #
#    the Free Software Foundation; either version 2 of the License, or     #
#    (at your option) any later version.                                   #
#                                                                          #
#    This program is distributed in the hope that it will be useful,       #
#    but WITHOUT ANY WARRANTY; without even the implied warranty of        #
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         #
#    GNU General Public License for more details.                          #
#                                                                          #
#    You should have received a copy of the GNU General Public License     #
#    along with this program; if not, write to the                         #
#    Free Software Foundation, Inc.,                                       #
#    59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             #
############################################################################


# --------------------------------------------------------------------------
# DESCRIPTION
#
# --------------------------------------------------------------------------

class InnerWindow {
	common bgcolor		{#DDDDDD}
	common active_titclr	{#DDDDFF}
	common inactive_titclr	{#DDDDDD}
	common title_bar_height	10

	private variable win_height
	private variable max_X				;# Int: Auxiliary variable for storing max. allowed position
	private variable max_Y				;# Int: Auxiliary variable for storing max. allowed position
	private variable close_cmd
	private variable title_bar			;# Widget: Window title bar
	private variable title_label			;# Widget: Label containg text "Scribble notepad"
	private variable close_button			;# Widget: Close button
	private variable coll_exp_but			;# Widget: Shade button
	private variable win
	private variable main_frame			;# Widget: Main window frame
	private variable minim_flag		0	;# Bool: Shaded or not
	private variable allow_raise_win	1	;# Bool: Allows to use command

	private variable click_X	;# Int: Auxiliary variable for storing last position
	private variable click_Y	;# Int: Auxiliary variable for storing last position


	constructor {path geometry title icon _close_cmd} {
		set max_X 1000
		set max_Y 1000
		
		set close_cmd $_close_cmd

		set win [frame $path -bd 1 -relief raised -bg $bgcolor -padx 2 -pady 2]
		set main_frame  [frame $win.main_frame]

		## Create title bar
		 # - Title bar frame
		set title_bar [frame $win.title_bar	\
			-bg $active_titclr		\
			-height $title_bar_height	\
		]
		set title_label [label $title_bar.text	\
			-bg $active_titclr		\
			-compound left -text $title	\
		]
		if {$icon != {}} {
			$title_label configure -image $icon
		}
		 # - Button "Close"
		set close_button [Button $title_bar.close_but		\
			-activebackground $active_titclr -pady 0	\
			-highlightthickness 0 -bd 0 -bg $active_titclr 	\
			-command "$this close" -overrelief raised	\
			-image ::ICONS::16::button_cancel		\
			-helptext "Close" -takefocus 0 -relief flat	\
		]
		setStatusTip -widget $close_button -text "Close"
		 # - Button "Shade"
		set coll_exp_but [Button $title_bar.col_exp_but			\
			-activebackground $active_titclr -pady 0		\
			-highlightthickness 0 -relief flat -takefocus 0		\
			-command "$this collapse_expand" -overrelief raised	\
			-image ::ICONS::16::_1uparrow -bd 0 			\
			-helptext "Shade" -bg $active_titclr			\
		]
		setStatusTip -widget $coll_exp_but -text "Shade"
		 # Pack buttons
		pack $coll_exp_but -padx 5 -side left -pady 0 -ipady 0
		pack $title_label -side left -fill x -pady 0 -ipady 0 -expand 1
		pack $close_button -side right -pady 0 -ipady 0 -padx 3
		 # Set title bar event bindings
		bind $title_label <Double-1> "$this collapse_expand"
		bind $title_label <Button-1> "$this title_B1 %X %Y; break"
		bind $title_label <B1-Motion> "$this title_B1_motion %X %Y"
		bind $title_label <ButtonRelease-1>  "$this title_B1_release"

		
		pack $title_bar -fill x
		pack $main_frame -fill both -expand 1

		# Show the window
		set x 0
		set y 0
		scan $geometry "=%dx%d+%d+%d" w h x y
		set win_height $h
		bind $win <Destroy> "delete object $this"
		bind $main_frame <Destroy> "delete object $this"
		bind $win <Visibility> "$this raise_win"
		bind $win <FocusIn> "$this focusin"
		bind $win <FocusOut> "$this focusout"
		place $win		\
			-x $x		\
			-y $y		\
			-width $w	\
			-height $h	\
			-anchor nw
		raise $win
	}

	public method withdraw {} {
		place forget $win
	}

	public method close {
		eval $close_cmd
		destroy $win
	}

	public method get_frame {} {
		return $main_frame
	}

	public method geometry args {
		if {$args != {}} {
			scan $args "=%dx%d+%d+%d" w h x y
			place $win		\
				-x $x		\
				-y $y		\
				-width $w	\
				-height $h	\
				-anchor nw
		}

		set result {=}
		append result	\
			[winfo width $win] {x} [winfo height $win] {+} [winfo x $win] {+} [winfo x $win]
		return $result
	}

	public method focusin {} {
# 		foreach widget [list $title_bar $title_label $close_button $coll_exp_but] {
# 			$widget configure -bg $active_titclr
# 		}
	}

	public method focusout {} {
# 		foreach widget [list $title_bar $title_label $close_button $coll_exp_but] {
# 			$widget configure -bg $inactive_titclr
# 		}
	}

	## (Un)Shade window
	 # @return void
	public method collapse_expand {} {
		# Object variables
		set minim_flag [expr {!$minim_flag}]

		# Shade
		if {$minim_flag} {
			set image _1downarrow
			pack forget $main_frame
			place $win -height [winfo height $win.title_bar]
		# Unshade
		} {
			set image _1uparrow
			pack $main_frame -fill both -expand 1
			place $win -height $win_height
		}
		$coll_exp_but configure -image ::ICONS::16::$image
	}


	## Insure window visibility
	 # @return void
	public method raise_win {} {
		if {!$allow_raise_win} {return}
		set allow_raise_win 0
		after 1000 "$this set_allow_raise_win"
		raise $win
	}

	## @see raise_win
	 # @return void
	public method set_allow_raise_win {} {
		set allow_raise_win 1
	}

	## Event handler: title bar <Button-1>
	 # @parm Int x - Absolute X coordinate
	 # @parm Int y - Absolute Y coordinate
	 # @return void
	public method title_B1 {x y} {
		set click_X [expr {[winfo x $win] - $x}]
		set click_Y [expr {[winfo y $win] - $y}]

		set max_X [winfo width .]
		set max_Y [winfo height .]
		incr max_X -70
		incr max_Y -70

		$title_bar	configure -cursor fleur
		$title_label	configure -cursor fleur
	}

	## Event handler: title bar <ButtonRelease-1>
	 # @return void
	public method title_B1_release {} {
		$title_bar	configure -cursor left_ptr
		$title_label	configure -cursor left_ptr
	}

	## Event handler: title bar <B1-Motion>
	 # @parm Int x - Absolute X coordinate
	 # @parm Int y - Absolute Y coordinate
	 # @return void
	public method title_B1_motion {x y} {
		incr x $click_X
		incr y $click_Y

		if {$x > 0 && $x < $max_X} {
			place $win -x $x
		}
		if {$y > 0 && $y < $max_Y} {
			place $win -y $y
		}
	}
}
