#!/usr/bin/tclsh
# Part of MCU 8051 IDE ( http://mcu8051ide.sf.net )

############################################################################
#    Copyright (C) 2007-2009 by Martin Ošmera                              #
#    martin.osmera@gmail.com                                               #
#                                                                          #
#    This program is free software; you can redistribute it and#or modify  #
#    it under the terms of the GNU General Public License as published by  #
#    the Free Software Foundation; either version 2 of the License, or     #
#    (at your option) any later version.                                   #
#                                                                          #
#    This program is distributed in the hope that it will be useful,       #
#    but WITHOUT ANY WARRANTY; without even the implied warranty of        #
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         #
#    GNU General Public License for more details.                          #
#                                                                          #
#    You should have received a copy of the GNU General Public License     #
#    along with this program; if not, write to the                         #
#    Free Software Foundation, Inc.,                                       #
#    59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             #
############################################################################


# --------------------------------------------------------------------------
# DESCRIPTION
#
# --------------------------------------------------------------------------

class LedPanel {
	inherit VirtualHWComponent

	common cb_font	[font create -weight bold -size -10 -family {helvetica}]
	common COMPONENT_NAME	"LED panel"
	common CLASS_NAME	"LedPanel"
	common COMPONENT_ICON	{ledpanel}

	# Configuration menu
	common CONFMENU {
		{cascade	{Color}			0	"colorize"	.color	false 1 {
			{radiobutton	"Red"		{}
				::LedPanel::color		{red}
				"color_changed"	0	""}
			{radiobutton	"Orange"	{}
				::LedPanel::color		{orange}
				"color_changed"	0	""}
			{radiobutton	"Yellow"	{}
				::LedPanel::color		{yellow}
				"color_changed"	0	""}
			{radiobutton	"Green"		{}
				::LedPanel::color		{green}
				"color_changed"	0	""}
			{radiobutton	"Blue"		{}
				::LedPanel::color		{blue}
				"color_changed"	0	""}
			{radiobutton	"Purple"	{}
				::LedPanel::color		{purple}
				"color_changed"	0	""}
		}}
		{separator}
		{command	{Show help}		{}	5	"show_help"	{help}
			"Show brief help"}
		{separator}
		{command	{Save configuration}	{}	0	"save_as"	{filesave}
			"Save configuration into a file"}
		{command	{Load configuration}	{}	0	"load_from"	{fileopen}
			"Load configuration from a file"}
	}
	

	private variable conf_led_color		{red}
	private variable leds
	private variable connection_port
	private variable connection_pin

	constructor {_project} {
		set component_name	$COMPONENT_NAME
		set class_name		$CLASS_NAME
		set component_icon	$COMPONENT_ICON
		
		set project $_project
		$project pale_register_output_device $this
		$project pale_set_modified

		array set connection_port	{0 - 1 - 2 - 3 - 4 - 5 - 6 - 7 -}
		array set connection_pin	{0 - 1 - 2 - 3 - 4 - 5 - 6 - 7 -}

		create_gui
		mcu_changed
		on_off [$project pale_is_enabled]

		for {set i 0} {$i < 8} {incr i} {
			$canvas_widget.cb_b$i setvalue first
			$canvas_widget.cb_p$i setvalue first
		}
	}

	destructor {
		$project pale_unregister_output_device $this
		destroy $win
	}

	public method reconnect {i} {
		set connection_port($i) [$canvas_widget.cb_p$i get]
		set connection_pin($i)	[$canvas_widget.cb_b$i get]
		if {$connection_pin($i) != {-}} {
			set connection_pin($i)	[expr {7 - $connection_pin($i)}]
		}

		if {$drawing_on} {
			new_state [$project pale_get_true_state]
		}
		set_modified
	}

	private method create_gui {} {
		set win [toplevel .ledpanel$count -class $component_name]

		set canvas_widget [canvas $win.canvas	\
			-bg white -width 0 -height 0	\
			-highlightthickness 0		\
		]

		set led_y 35
		set cb_p_y 65
		set cb_b_y 85
		set usr_n_y 105
		set x 50

		$canvas_widget create text 5 $cb_p_y	\
			-text {PORT}			\
			-font $cb_font			\
			-anchor w
		$canvas_widget create text 5 $cb_b_y	\
			-text {BIT}			\
			-font $cb_font			\
			-anchor w
		$canvas_widget create text 5 $usr_n_y	\
			-text {Note}			\
			-font $cb_font			\
			-anchor w
		$canvas_widget create window 208 $usr_n_y	\
			-window [entry $canvas_widget.usr_note	\
				-bg white -validate all		\
				-vcmd "$this set_modified"	\
			]	\
			-width 330
		bindtags $canvas_widget.usr_note	\
			[list $canvas_widget.usr_note Entry $win all .]

		for {set i 0} {$i < 8} {incr i} {
			set leds($i) [$canvas_widget create image $x $led_y	\
				-image ::ICONS::16::ledgray		\
			]

			incr x 6

			$canvas_widget create window $x $cb_p_y -anchor center	\
				-window [ComboBox $canvas_widget.cb_p$i	\
					-width 1 -font $cb_font		\
					-modifycmd "$this reconnect $i"	\
					-entrybg white -editable 0	\
					-hottrack 1			\
				]
			$canvas_widget create window $x $cb_b_y -anchor center	\
				-window [ComboBox $canvas_widget.cb_b$i	\
					-width 1 -font $cb_font		\
					-modifycmd "$this reconnect $i"	\
					-values {- 0 1 2 3 4 5 6 7}	\
					-entrybg white -editable 0	\
					-hottrack 1			\
				]

			catch {
				bindtags $canvas_widget.cb_p$i.e	\
					[list $canvas_widget.cb_p$i.e Entry all .]
				bindtags $canvas_widget.cb_b$i.e	\
					[list $canvas_widget.cb_b$i.e Entry all .]
			}

			incr x 6
			draw_led $x 10

			incr x 30
		}
		
		set start_stop_button [Button $canvas_widget.start_stop_button	\
			-bd 1 -command "$this on_off_button_press"		\
			-padx 0 -pady 0 -highlightthickness 0 -width 3		\
			-helptext "Turn HW simulation on/off"			\
		]
		setStatusTip -widget $start_stop_button -text "Turn HW simulation on/off"
		bind $start_stop_button <Button-3> "$this on_off_button_press; break"
		$canvas_widget create window 22 2 -window $start_stop_button -anchor nw
		
		set conf_button [button $canvas_widget.conf_but	\
			-image ::ICONS::16::configure -bd 0	\
			-command "$this config_menu"		\
			-relief flat -overrelief raised		\
			-bg white -activebackground white	\
			-highlightthickness 0			\
		]
		setStatusTip -widget $conf_button -text "Configure"
		$canvas_widget create window 20 2 -window $conf_button -anchor ne
		
		pack $canvas_widget -fill both -expand 1
		
		# Set window parameters
		wm geometry $win =380x120
		wm iconphoto $win ::ICONS::16::$component_icon
		wm title $win "$component_name - [string trim $project {:}] - MCU 8051 IDE"
		wm resizable $win 0 0
		wm protocol $win WM_DELETE_WINDOW "$this close_window"
		
		bindtags $win [list $win Toplevel all .]
	}

	private method draw_led {x y} {
		$canvas_widget create line			\
			[expr {4 + $x}]		[expr {2 + $y}]	\
			[expr {6 + $x}]		[expr {0 + $y}]	\
			[expr {8 + $x}]		[expr {2 + $y}]	\
			[expr {6 + $x}]		[expr {0 + $y}]	\
			[expr {6 + $x}]		[expr {6 + $y}]	\
			[expr {8 + $x}]		[expr {6 + $y}]	\
			[expr {8 + $x}]		[expr {17 + $y}]\
			[expr {4 + $x}]		[expr {17 + $y}]\
			[expr {4 + $x}]		[expr {6 + $y}]	\
			[expr {6 + $x}]		[expr {6 + $y}]	\
			[expr {4 + $x}]		[expr {6 + $y}]	\
			[expr {4 + $x}]		[expr {17 + $y}]\
			[expr {6 + $x}]		[expr {17 + $y}]\
			[expr {6 + $x}]		[expr {37 + $y}]\
			[expr {6 + $x}]		[expr {28 + $y}]\
			[expr {0 + $x}]		[expr {28 + $y}]\
			[expr {12 + $x}]	[expr {28 + $y}]\
			[expr {6 + $x}]		[expr {28 + $y}]\
			[expr {6 + $x}]		[expr {27 + $y}]\
			[expr {0 + $x}]		[expr {21 + $y}]\
			[expr {12 + $x}]	[expr {21 + $y}]\
			[expr {6 + $x}]		[expr {27 + $y}]\
			[expr {6 + $x}]		[expr {37 + $y}]\
			[expr {0 + $x}]		[expr {43 + $y}]\
			-fill {#888888}

		$canvas_widget create line			\
			[expr {14 + $x}]	[expr {22 + $y}]	\
			[expr {17 + $x}]	[expr {19 + $y}]	\
			[expr {15 + $x}]	[expr {19 + $y}]	\
			[expr {17 + $x}]	[expr {19 + $y}]	\
			[expr {17 + $x}]	[expr {21 + $y}]	\
			-fill {#888888}

		$canvas_widget create line			\
			[expr {14 + $x}]	[expr {26 + $y}]	\
			[expr {17 + $x}]	[expr {23 + $y}]	\
			[expr {15 + $x}]	[expr {23 + $y}]	\
			[expr {17 + $x}]	[expr {23 + $y}]	\
			[expr {17 + $x}]	[expr {25 + $y}]	\
			-fill {#888888}
	}

	private method which_port_pin {i} {
		return [list $connection_port($i) $connection_pin($i)]
	}

	public method on_off_button_press {} {
		$project pale_all_on_off
	}

	# ------------------------------------------------------------------
	# VIRTUAL HW COMMON INTERFACE
	# ------------------------------------------------------------------

	public method mcu_changed {} {
		set avaliable_ports [concat - [$project pale_get_avaliable_ports]]
		
		for {set i 0} {$i < 8} {incr i} {
			$canvas_widget.cb_p$i configure -values $avaliable_ports

			if {[lsearch -ascii -exact $avaliable_ports $connection_port($i)] == -1} {
				$canvas_widget.cb_p$i setvalue first
				set connection_port($i) {-}
			}
		}
	}

	public method on_off {state} {
		set drawing_on $state
		
		if {$state} {
			$start_stop_button configure	-text {ON}			\
				-bg {#AAFFAA}		-activebackground {#DDFFDD}	\
				-fg {#000000}		-activeforeground {#00FF00}

			new_state [$project pale_get_true_state]

		} else {
			$start_stop_button configure	-text {OFF}			\
				-bg {#FFAAAA}		-activebackground {#FFDDDD}	\
				-fg {#000000}		-activeforeground {#FF0000}

			for {set i 0} {$i < 8} {incr i} {
				$canvas_widget itemconfigure $leds($i)	\
					-image ::ICONS::16::ledgray
			}
		}
	}

	public method new_state {state} {
		for {set i 0} {$i < 8} {incr i} {
			set pp [which_port_pin $i]
			if {[lindex $pp 0] == {-} || [lindex $pp 1] == {-}} {
				$canvas_widget itemconfigure $leds($i) -image ::ICONS::16::ledgray
				continue
			}
			
			switch -- [lindex $state $pp] {
				{0} {set image led${conf_led_color}}
				{1} {set image ledgray}
				{?} {set image ledgray}
				{=} {set image ledgray}
				{} {set image ledgray}
				default {set image ledgray${conf_led_color}}
			}
			$canvas_widget itemconfigure $leds($i) -image ::ICONS::16::$image
		}
	}

	public method withdraw_window {} {
		wm withdraw $win
	}

	public method get_config {} {
		return [list		\
			$class_name	\
			[list		\
				[array get connection_port]	\
				[array get connection_pin]	\
				[wm geometry $win]		\
				[$canvas_widget.usr_note get]	\
				$conf_led_color			\
			]	\
		]
	}

	public method set_config {state} {
		if {[catch {
			array set connection_port [lindex $state 0]
			array set connection_pin [lindex $state 1]
			if {[string length [lindex $state 2]]} {
				wm geometry $win [lindex $state 2]
			}
			
			$canvas_widget.usr_note delete 0
			$canvas_widget.usr_note insert 0 [lindex $state 3]
			
			set conf_led_color [lindex $state 4]
		
			for {set i 0} {$i < 8} {incr i} {
				set pin $connection_pin($i)
				if {$pin != {-}} {
					set pin	[expr {7 - $pin}]
				}
				set idx [lsearch -ascii -exact			\
					[$canvas_widget.cb_b$i cget -values]	\
					$pin					\
				]
				if {$idx == -1} {
					set idx 0
				}
				$canvas_widget.cb_b$i setvalue @$idx


				set idx [lsearch -ascii -exact			\
					[$canvas_widget.cb_p$i cget -values]	\
					$connection_port($i)			\
				]
				if {$idx == -1} {
					set idx 0
				}
				$canvas_widget.cb_p$i setvalue @$idx
			}

			new_state [$project pale_get_true_state]
			update

		}]} then {
			puts "Unable to load config for $class_name"
			return 0
		} else {
			clear_modified
			return 1
		}
	}
	
	public method reset {} {
		new_state [$project pale_get_true_state]
	}

	# ------------------------------------------------------

	public method config_menu_special {} {
		set ::${class_name}::color $conf_led_color
	}

	public method create_config_menu_special {} {
		foreach item {	Red	Orange	Yellow	Green	Blue	Purple	} \
			color {	#DD0000	#DD8800	#DDDD00	#00DD00	#0000DD	#8800DD	} \
		{
			$conf_menu.color entryconfigure $item -foreground $color
		}
	}
	
	public method color_changed {} {
		set conf_led_color ${::LedPanel::color}
	
		if {$drawing_on} {
			new_state [$project pale_get_true_state]
		}
		set_modified
	}

	public method show_help_special {text_widget} {
# 		$text_widget insert insert "How to use this tool:\n"
# 		$text_widget tag add tag_bold insert-1l insert
# 		$text_widget insert insert "   1) Connect programmer to the computer\n"
# 		$text_widget tag add tag_bold insert-1l {insert-1l+6c}
# 		$text_widget insert insert "   2) Select port in section \"Programmer\"\n"
# 		$text_widget tag add tag_bold insert-1l {insert-1l+6c}
# 		$text_widget insert insert "   3) Click on button \"Autodetect device\"\n"
# 		$text_widget tag add tag_bold insert-1l {insert-1l+6c}
# 		$text_widget insert insert "   4) If programmer is operational you will see name of connected MCU in section \"Connected MCU\"\n"
# 		$text_widget tag add tag_bold insert-1l {insert-1l+6c}
# 		$text_widget insert insert "   5) Now you can perform arbitrary desired operation\n"
# 		$text_widget tag add tag_bold insert-1l {insert-1l+6c}
# 		$text_widget insert insert "\nMost useful functions are upload flash memory in section \"Flash memory (remember than flash must be first erased before programming). And \"Reprogram\" in section Special functions\", it will erase and reprogram all on-chip memory."
	}

	public method close_window_special {} {
	}
}
