#!/usr/bin/tclsh
# Part of MCU 8051 IDE ( http://mcu8051ide.sf.net )

############################################################################
#    Copyright (C) 2007-2009 by Martin Ošmera                              #
#    martin.osmera@gmail.com                                               #
#                                                                          #
#    This program is free software; you can redistribute it and#or modify  #
#    it under the terms of the GNU General Public License as published by  #
#    the Free Software Foundation; either version 2 of the License, or     #
#    (at your option) any later version.                                   #
#                                                                          #
#    This program is distributed in the hope that it will be useful,       #
#    but WITHOUT ANY WARRANTY; without even the implied warranty of        #
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         #
#    GNU General Public License for more details.                          #
#                                                                          #
#    You should have received a copy of the GNU General Public License     #
#    along with this program; if not, write to the                         #
#    Free Software Foundation, Inc.,                                       #
#    59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             #
############################################################################


# --------------------------------------------------------------------------
# DESCRIPTION
#
# --------------------------------------------------------------------------

class MatrixKeyPad {
	inherit VirtualHWComponent

	common cb_font [font create	\
		-weight bold		\
		-size -10		\
		-family {helvetica}	\
	]
	common cb_font_n [font create	\
		-size -10		\
		-family {helvetica}	\
	]
	
	common COMPONENT_NAME	"Matrix Keypad"
	common CLASS_NAME	"MatrixKeyPad"
	common COMPONENT_ICON	{matrixkeypad}
  
	# Configuration menu
	common CONFMENU {
		{checkbutton	"Radio buttons"		{}	{::MatrixKeyPad::menu_radio_buttons}
			1 0 0	{value_radio_buttons_changed}
			""}
		{separator}
		{command	{Show help}		{}	5	"show_help"	{help}
			"Show brief help"}
		{separator}
		{command	{Save configuration}	{}	0	"save_as"	{filesave}
			"Save configuration into a file"}
		{command	{Load configuration}	{}	0	"load_from"	{fileopen}
			"Load configuration from a file"}
	}

	private variable radio_buttons
	private variable keys
	private variable wire
	private variable wire_o
	private variable rect
	private variable lever
	private variable text
	private variable lines_o
	private variable lines

	private variable row_wire
	private variable col_wire
	
	private variable connection_port
	private variable connection_pin
	private variable enaged

	
	constructor {_project} {
		set component_name	$COMPONENT_NAME
		set class_name		$CLASS_NAME
		set component_icon	$COMPONENT_ICON
		
		set project $_project
		$project pale_register_input_device $this
		$project pale_set_modified

		set radio_buttons 1
		array set connection_port	{0 - 1 - 2 - 3 - 4 - 5 - 6 - 7 -}
		array set connection_pin	{0 - 1 - 2 - 3 - 4 - 5 - 6 - 7 -}
		array set keys			{
			0 0 1 0 2 0 3 0
			4 0 5 0 6 0 7 0
			8 0 9 0 10 0 11 0
			12 0 13 0 14 0 15 0
		}

		for {set port 0} {$port < 5} {incr port} {
			for {set bit 0} {$bit < 8} {incr bit} {
				set enaged($port,$bit) 0
			}
		}

		create_gui
		mcu_changed
		on_off [$project pale_is_enabled]

		for {set i 0} {$i < 8} {incr i} {
			$canvas_widget.cb_b$i setvalue first
			$canvas_widget.cb_p$i setvalue first
		}
	}

	destructor {
		$project pale_unregister_input_device $this
		destroy $win
	}

	private method evaluete_enaged_pins {} {
		for {set port 0} {$port < 5} {incr port} {
			for {set bit 0} {$bit < 8} {incr bit} {
				if {$enaged($port,$bit)} {
					$project pale_disengage_pin_by_input_device $port $bit $this
					set enaged($port,$bit) 0
				}
			}
		}

		for {set i 0} {$i < 8} {incr i} {
			set port $connection_port($i)
			set bit $connection_pin($i)

			if {$port == {-} || $bit == {-}} {
				continue
			}

			set enaged($port,$bit) 1
		}
		
		for {set port 0} {$port < 5} {incr port} {
			for {set bit 0} {$bit < 8} {incr bit} {
				if {$enaged($port,$bit)} {
					$project pale_engage_pin_by_input_device $port $bit $this
				}
			}
		}
	}

	public method reconnect {i} {
		set connection_port($i) [$canvas_widget.cb_p$i get]
		set connection_pin($i)	[$canvas_widget.cb_b$i get]
		if {$connection_pin($i) != {-}} {
			set connection_pin($i)	[expr {7 - $connection_pin($i)}]
		}

		evaluete_enaged_pins

		if {$drawing_on} {
			$project pale_reevaluate_IO
		}
		set_modified
	}

	private method create_gui {} {
		set win [toplevel .matrixkeypad$count -class $component_name]

		set canvas_widget [canvas $win.canvas	\
			-bg white -width 0 -height 0	\
			-highlightthickness 0		\
		]

		$canvas_widget create text 33 20	\
			-text {PORT}			\
			-font $cb_font			\
			-anchor e
		$canvas_widget create text 35 20	\
			-text {BIT}			\
			-font $cb_font			\
			-anchor w

		$canvas_widget create text 80 175	\
			-text {PORT}			\
			-font $cb_font			\
			-anchor e
		$canvas_widget create text 80 195	\
			-text {BIT}			\
			-font $cb_font			\
			-anchor e
 
		$canvas_widget create text 5 220	\
			-text {Note}			\
			-font $cb_font			\
			-anchor w
		$canvas_widget create window 35 220		\
			-window [entry $canvas_widget.usr_note	\
				-bg white -validate all		\
				-vcmd "$this set_modified"	\
			]	\
			-width 180 -anchor w
		bindtags $canvas_widget.usr_note	\
			[list $canvas_widget.usr_note Entry $win all .]

		set sep 37
		set x_0 65
		set y 5
		set i 0
		set x $x_0
		for {set row 0} {$row < 4} {incr row} {
			for {set col 0} {$col < 4} {incr col} {
				draw_key $x $y $i
				incr i
				incr x $sep
			}
			incr y $sep
			set x $x_0
		}
		draw_col_row_wires -10 -25

		set x 30
		set y 40
		for {set i 0} {$i < 4} {incr i} {
			$canvas_widget create window $x $y -anchor e	\
				-window [ComboBox $canvas_widget.cb_p$i	\
					-width 1 -font $cb_font		\
					-modifycmd "$this reconnect $i"	\
					-entrybg white -editable 0	\
					-hottrack 1			\
				]
			$canvas_widget create window $x $y -anchor w	\
				-window [ComboBox $canvas_widget.cb_b$i	\
					-width 1 -font $cb_font		\
					-modifycmd "$this reconnect $i"	\
					-values {- 0 1 2 3 4 5 6 7}	\
					-entrybg white -editable 0	\
					-hottrack 1			\
				]

			catch {
				bindtags $canvas_widget.cb_p$i.e	\
					[list $canvas_widget.cb_p$i.e Entry all .]
				bindtags $canvas_widget.cb_b$i.e	\
					[list $canvas_widget.cb_b$i.e Entry all .]
			}
			
			incr y $sep
		}

		set cb_p_y 175
		set cb_b_y 195
		set x 95
		for {set i 4} {$i < 8} {incr i} {
			$canvas_widget create window $x $cb_p_y -anchor center	\
				-window [ComboBox $canvas_widget.cb_p$i	\
					-width 1 -font $cb_font		\
					-modifycmd "$this reconnect $i"	\
					-entrybg white -editable 0	\
					-hottrack 1			\
				]
			$canvas_widget create window $x $cb_b_y -anchor center	\
				-window [ComboBox $canvas_widget.cb_b$i	\
					-width 1 -font $cb_font		\
					-modifycmd "$this reconnect $i"	\
					-values {- 0 1 2 3 4 5 6 7}	\
					-entrybg white -editable 0	\
					-hottrack 1			\
				]

			catch {
				bindtags $canvas_widget.cb_p$i.e	\
					[list $canvas_widget.cb_p$i.e Entry all .]
				bindtags $canvas_widget.cb_b$i.e	\
					[list $canvas_widget.cb_b$i.e Entry all .]
			}

			incr x $sep
		}
		
		set start_stop_button [Button $canvas_widget.start_stop_button	\
			-bd 1 -command "$this on_off_button_press"		\
			-padx 0 -pady 0 -highlightthickness 0 -width 3		\
			-helptext "Turn HW simulation on/off"			\
		]
		setStatusTip -widget $start_stop_button -text "Turn HW simulation on/off"
		bind $start_stop_button <Button-3> "$this on_off_button_press; break"
		$canvas_widget create window 22 190 -window $start_stop_button -anchor w
		
		set conf_button [button $canvas_widget.conf_but	\
			-image ::ICONS::16::configure -bd 0	\
			-command "$this config_menu"		\
			-relief flat -overrelief raised		\
			-bg white -activebackground white	\
			-highlightthickness 0			\
		]
		setStatusTip -widget $conf_button -text "Configure"
		$canvas_widget create window 20 190 -window $conf_button -anchor e

		pack $canvas_widget -fill both -expand 1

		# Set window parameters
		wm geometry $win =225x235
		wm iconphoto $win ::ICONS::16::$component_icon
		wm title $win "$component_name - [string trim $project {:}] - MCU 8051 IDE"
		wm resizable $win 0 0
		wm protocol $win WM_DELETE_WINDOW "$this close_window"
		
		bindtags $win [list $win Toplevel all .]
	}

	private method draw_col_row_wires {x_0 y_0} {

		set x $x_0
		set y $y_0
 		for {set i 0} {$i < 4} {incr i} {
 			set col_wire($i) [list]
 			
			lappend col_wire($i) [$canvas_widget create line\
				[expr {$x + 107}] [expr {$y + 37}]	\
				[expr {$x + 107}] [expr {$y + 190}]	\
				-fill #000000 -width 1			\
			]

			for {set j 0} {$j < 4} {incr j} {
				lappend col_wire($i) [$canvas_widget create line\
					[expr {$x + 100}] [expr {$y + 37}]	\
					[expr {$x + 107}] [expr {$y + 37}]	\
					-fill #000000 -width 1			\
				]
				if {$j != 0} {
					lappend col_wire($i) [$canvas_widget create oval\
						[expr {$x + 105}] [expr {$y + 35}]	\
						[expr {$x + 109}] [expr {$y + 39}]	\
						-fill #000000 -width 0			\
					]
				}
				incr y 37
			}
			set y $y_0
			incr x 37
		}

		set x $x_0
		set y $y_0
 		for {set i 0} {$i < 4} {incr i} {
 			set row_wire($i) [list]
 			
			lappend row_wire($i) [$canvas_widget create line\
				[expr {$x + 200}] [expr {$y + 64}]	\
				[expr {$x + 30}] [expr {$y + 64}]	\
				-fill #000000 -width 1			\
			]

			for {set j 0} {$j < 4} {incr j} {
				lappend row_wire($i) [$canvas_widget create line\
					[expr {$x + 88}] [expr {$y + 60}]	\
					[expr {$x + 88}] [expr {$y + 64}]	\
					-fill #000000 -width 1			\
				]
				if {$j == 3} {
					break
				}

				lappend row_wire($i) [$canvas_widget create oval\
					[expr {$x + 86}] [expr {$y + 62}]	\
					[expr {$x + 90}] [expr {$y + 66}]	\
					-fill #000000 -width 0			\
				]
				incr x 37
			}
			set x $x_0
			incr y 37
		}
	}

	public method key_click {i} {
		set keys($i) [expr {!$keys($i)}]
		key_state_changed $i

		if {$radio_buttons} {
			for {set j 0} {$j < 16} {incr j} {
				if {$j == $i} {
					continue
				}
				if {$keys($j)} {
					set keys($j) 0
					key_state_changed $j
				}
			}
		}

 		$project pale_reevaluate_IO
		set_modified
	}

	private method key_state_changed {i} {
		if {$keys($i)} {
			$canvas_widget itemconfigure $lever(0$i) -fill #FFFFFF
			$canvas_widget itemconfigure $lever(1$i) -fill #000000
			$canvas_widget itemconfigure $text($i) -font $cb_font
			$canvas_widget itemconfigure $rect($i) -outline #333333 -width 2
		} {
			$canvas_widget itemconfigure $lever(0$i) -fill #000000
			$canvas_widget itemconfigure $lever(1$i) -fill #FFFFFF
			$canvas_widget itemconfigure $text($i) -font $cb_font_n
			$canvas_widget itemconfigure $rect($i) -outline #CCCCCC -width 1
		}
	}

	public method key_leave {i} {
		if {$keys($i)} {
			set color {#333333}
		} {
			set color {#CCCCCC}
		}
		$canvas_widget itemconfigure $rect($i) -outline $color
	}

	public method key_enter {i} {
		$canvas_widget itemconfigure $rect($i) -outline {#0000FF}
	}

	private method draw_key {x y i} {
		set rect($i) [$canvas_widget create rectangle	\
			[expr {$x + 1}] [expr {$y + 1}]		\
			[expr {$x + 25}] [expr {$y + 29}]	\
			-width 1 -outline #CCCCCC -fill #FFFFFF	\
		]

		set text($i) [$canvas_widget create text 	\
			[expr {$x + 20}] [expr {$y + 15}]	\
			-font $cb_font_n			\
			-text [lindex {1 2 3 A 4 5 6 B 7 8 9 C * 0 # D} $i]	\
		]

		set lever(1$i) [$canvas_widget create line	\
			[expr {$x + 11}] [expr {$y + 22}]	\
			[expr {$x + 11}] [expr {$y + 6}]	\
			-width 1 -fill #FFFFFF			\
		]	
		set lever(0$i) [$canvas_widget create line	\
			[expr {$x + 10}] [expr {$y + 22}]	\
			[expr {$x + 5}] [expr {$y + 6}]		\
			-width 1 -fill #000000			\
		]

		set lines($i) [$canvas_widget create line	\
			[expr {$x + 16}] [expr {$y + 7}]	\
			[expr {$x + 25}] [expr {$y + 7}]	\
			-width 1 -fill #000000			\
		]

		set lines_o($i) [$canvas_widget create oval	\
			[expr {$x + 11}] [expr {$y + 5}]	\
			[expr {$x + 15}] [expr {$y + 9}]	\
			-width 1 -outline #000000		\
		]

		set wire($i) [$canvas_widget create line	\
			[expr {$x + 13}] [expr {$y + 26}]	\
			[expr {$x + 13}] [expr {$y + 30}]	\
			-width 1 -fill #000000			\
		]

		set wire_o($i) [$canvas_widget create oval 	\
			[expr {$x + 11}] [expr {$y + 21}]	\
			[expr {$x + 15}] [expr {$y + 25}]	\
			-width 1 -outline #000000		\
		]

		foreach items [list					\
			$rect($i)	$lines_o($i)	$wire_o($i)	\
			$lever(0$i)	$lever(1$i)	$text($i)	\
			$lines($i)	$wire($i)			\
		] {
			foreach item $items {
				$canvas_widget bind $item <Enter> "$this key_enter $i"
				$canvas_widget bind $item <Leave> "$this key_leave $i"
				$canvas_widget bind $item <Button-1> "$this key_click $i"
			}
		}
	}

	private method which_port_pin {i} {
		return [list $connection_port($i) $connection_pin($i)]
	}

	public method on_off_button_press {} {
		$project pale_all_on_off
	}

	private method which_color {state} {
		switch -- $state {
			{} {
				return {#000000}
			}
			{0} {
				return {#00FF00}
			}
			{1} {
				return {#FF0000}
			}
			{=} {
				return {#FF00AA}
			}
			{?} {
				return {#888888}
			}
			default {
				return {#FF8800}
			}
		}
	}

	# ------------------------------------------------------------------
	# VIRTUAL HW COMMON INTERFACE
	# ------------------------------------------------------------------

	public method mcu_changed {} {
		set avaliable_ports [concat - [$project pale_get_avaliable_ports]]
		
		for {set i 0} {$i < 8} {incr i} {
			$canvas_widget.cb_p$i configure -values $avaliable_ports

			if {[lsearch -ascii -exact $avaliable_ports $connection_port($i)] == -1} {
				$canvas_widget.cb_p$i setvalue first
				set connection_port($i) {-}
			}
		}
	}

	public method on_off {state} {
		set drawing_on $state
		
		if {$state} {
			$start_stop_button configure	-text {ON}			\
				-bg {#AAFFAA}		-activebackground {#DDFFDD}	\
				-fg {#000000}		-activeforeground {#00FF00}

		} else {
			$start_stop_button configure	-text {OFF}			\
				-bg {#FFAAAA}		-activebackground {#FFDDDD}	\
				-fg {#000000}		-activeforeground {#FF0000}
		}
		
		new_state [$project pale_get_true_state]
	}

	public method new_state {state} {
		set row_state [list {} {} {} {}]
		set col_state [list {} {} {} {}]

		for {set i 0} {$i < 8} {incr i} {
			set pp [which_port_pin $i]
			if {[lindex $pp 0] == {-} || [lindex $pp 1] == {-}} {
				continue
			}

			# Rows
			if {$i < 4} {
				lset row_state $i [lindex $state $pp]
			# Columns
			} {
				lset col_state [expr {$i - 4}] [lindex $state $pp]
			}
		}

		set r_state {}
		set c_state {}
		set new {}
		set changes 1
		while {$changes} {
			set k_i 0
			for {set r 0} {$r < 4} {incr r} {
				for {set c 0} {$c < 4} {incr c} {
					if {$keys($k_i)} {
						set r_state [lindex $row_state $r]
						set c_state [lindex $col_state $c]

						set new [$project pale_combine_values	\
							$r_state $c_state		\
						]

						if {
							$new != $r_state
								||
							$new != $c_state
						} then {
							incr changes
						}

						lset row_state $r $new
						lset col_state $c $new
					}
					incr k_i
				}
			}

			incr changes -1
		}

		for {set i 0} {$i < 8} {incr i} {
			set pp [which_port_pin $i]
			if {[lindex $pp 0] == {-} || [lindex $pp 1] == {-}} {
				continue
			}

			# Rows
			if {$i < 4} {
				lset state $pp [lindex $row_state $i]
			# Columns
			} {
				lset state $pp [lindex $col_state [expr {$i - 4}]]
			}
		}

		for {set i 0} {$i < 4} {incr i} {
			set color [which_color [lindex $row_state $i]]
			foreach item $row_wire($i) {
				$canvas_widget itemconfigure $item -fill $color
			}
		}

		for {set i 0} {$i < 4} {incr i} {
			set color [which_color [lindex $col_state $i]]
			foreach item $col_wire($i) {
				$canvas_widget itemconfigure $item -fill $color
			}
		}

		return $state
	}

	public method withdraw_window {} {
		wm withdraw $win
	}

	public method get_config {} {
		return [list		\
			$class_name	\
			[list		\
				[array get connection_port]	\
				[array get connection_pin]	\
				[wm geometry $win]		\
				[$canvas_widget.usr_note get]	\
				[array get keys]		\
				$radio_buttons			\
			]	\
		]
	}

	public method set_config {state} {
		if {[catch {
			array set connection_port [lindex $state 0]
			array set connection_pin [lindex $state 1]
			if {[string length [lindex $state 2]]} {
				wm geometry $win [lindex $state 2]
			}

			$canvas_widget.usr_note delete 0
			$canvas_widget.usr_note insert 0 [lindex $state 3]
			
			array set keys [lindex $state 4]
			set radio_buttons [lindex $state 5]
		
			for {set i 0} {$i < 8} {incr i} {
				set pin $connection_pin($i)
				if {$pin != {-}} {
					set pin	[expr {7 - $pin}]
				}
				set idx [lsearch -ascii -exact			\
					[$canvas_widget.cb_b$i cget -values]	\
					$pin					\
				]
				if {$idx == -1} {
					set idx 0
				}
				$canvas_widget.cb_b$i setvalue @$idx


				set idx [lsearch -ascii -exact			\
					[$canvas_widget.cb_p$i cget -values]	\
					$connection_port($i)			\
				]
				if {$idx == -1} {
					set idx 0
				}
				$canvas_widget.cb_p$i setvalue @$idx
			}

			for {set i 0} {$i < 16} {incr i} {
				key_state_changed $i
			}

			evaluete_enaged_pins
			$project pale_reevaluate_IO
			update

		}]} then {
			puts "Unable to load config for $class_name"
			return 0
		} else {
			clear_modified
			return 1
		}
	}
	
	public method reset {} {
 		$project pale_reevaluate_IO
	}

	public method value_radio_buttons_changed {} {
		set radio_buttons $::MatrixKeyPad::menu_radio_buttons
	}

	# ------------------------------------------------------

	public method config_menu_special {} {
		set ::MatrixKeyPad::menu_radio_buttons $radio_buttons
	}

	public method create_config_menu_special {} {
	}

	public method show_help_special {text_widget} {
# 		$text_widget insert insert "How to use this tool:\n"
# 		$text_widget tag add tag_bold insert-1l insert
# 		$text_widget insert insert "   1) Connect programmer to the computer\n"
# 		$text_widget tag add tag_bold insert-1l {insert-1l+6c}
# 		$text_widget insert insert "   2) Select port in section \"Programmer\"\n"
# 		$text_widget tag add tag_bold insert-1l {insert-1l+6c}
# 		$text_widget insert insert "   3) Click on button \"Autodetect device\"\n"
# 		$text_widget tag add tag_bold insert-1l {insert-1l+6c}
# 		$text_widget insert insert "   4) If programmer is operational you will see name of connected MCU in section \"Connected MCU\"\n"
# 		$text_widget tag add tag_bold insert-1l {insert-1l+6c}
# 		$text_widget insert insert "   5) Now you can perform arbitrary desired operation\n"
# 		$text_widget tag add tag_bold insert-1l {insert-1l+6c}
# 		$text_widget insert insert "\nMost useful functions are upload flash memory in section \"Flash memory (remember than flash must be first erased before programming). And \"Reprogram\" in section Special functions\", it will erase and reprogram all on-chip memory."
	}

	public method close_window_special {} {
	}
}
